/*
	BeFree.cpp
	
	(c) 1996 Tim Stack
*/

#ifndef BEFREE_WINDOW_H
#include "BeFreeWindow.h"
#endif

#ifndef BEFREE_H
#include "BeFree.h"
#endif

main()
{
	BeFreeApplication *myapp;
	
	myapp = new BeFreeApplication();
	myapp->Run();
	myapp->FreeVols();
	delete myapp;
	return( 0 );
}

BeFreeApplication::BeFreeApplication() : BApplication( 'BFRE' )
{
	BStatusBar *sbar;
	BRect rect;
	float width;
	long index, count;
	char name[B_OS_NAME_LENGTH];
	char size[20];
	float cap, minw = 0;
	unsigned char colindex;
	system_info sysinfo;
	BView *top;
	
	get_system_info( &sysinfo );
	ScanVols();
	count = vols.CountItems();
	get_dock_width( &width );
	rect.Set( width, 0, 200, 100 );
	win = new BeFreeWindow( rect );
	rect = win->Bounds();
	rect.Set( XGUTTER, 0, rect.Width() - XGUTTER, 10 );
	win->Lock();
	sprintf( size, "%.1f", ((sysinfo.max_pages - sysinfo.used_pages) * 4096.0)/1024.0 );
	sbar = new BStatusBar( rect, "Memory", "Memory", "K Free" );
	win->AddChild( sbar );
	colindex = index_for_color( 200, 200, 200 );
	sbar->SetViewColor( system_colors()->color_list[colindex] );
	sbar->SetLowColor( system_colors()->color_list[colindex] );
	sbar->SetMaximum( sysinfo.max_pages );
	sbar->Progress( sysinfo.used_pages, 0, size );
	sbar->SetResizingMode( B_FOLLOW_LEFT_RIGHT );
	sbar->SetFlags( B_FULL_UPDATE_ON_RESIZE );
	if( (sbar->StringWidth( "Memory" ) + sbar->StringWidth( size ) + 10) > minw )
		minw = sbar->StringWidth( "Memory" ) + sbar->StringWidth( size ) + 10;
	rect = sbar->Bounds();
	height = rect.Height();
	rect.OffsetBy( XGUTTER, height );
	for( index = 1; index < count; index++, rect.OffsetBy( XGUTTER, 2 * height ) )
	{
		((BVolume *)vols.ItemAt( index ))->GetName( name );
		sprintf( size, "%.1f", (((BVolume *)vols.ItemAt( index ))->FreeBytes() / 1024.0) );
		sbar = new BStatusBar( rect, name, name, "K Free" );
		win->AddChild( sbar );
		sbar->SetViewColor( system_colors()->color_list[colindex] );
		sbar->SetLowColor( system_colors()->color_list[colindex] );
		sbar->SetMaximum( cap = (((BVolume *)vols.ItemAt( index ))->Capacity() / 1024.0) );
		sbar->Progress( cap );
		sbar->Progress( -(((BVolume *)vols.ItemAt( index ))->FreeBytes() / 1024.0), 0, size );
		sbar->SetResizingMode( B_FOLLOW_LEFT_RIGHT );
		sbar->SetFlags( B_FULL_UPDATE_ON_RESIZE );
		if( (sbar->StringWidth( name ) + sbar->StringWidth( size ) + 10) > minw )
			minw = sbar->StringWidth( name ) + sbar->StringWidth( size ) + 10;
		rect = sbar->Bounds();
	}
	win->ResizeTo( 200, height * count );
	win->SetSizeLimits( minw + 2 * XGUTTER, 32000.0, 0.0, 32000.0 );
	win->Unlock();
	SetPulseRate( 2000000 );
	win->Show();
}

void BeFreeApplication::AboutRequested()
{
	BAlert *alert;
	
	alert = new BAlert( "BeFree", "BeFree\nby Tim Stack\n\nA Utility to show\nfree space on volumes", "OK" );
	alert->Go();
}

void BeFreeApplication::VolumeMounted( long id )
{
	BVolume *vol;
	char name[B_OS_NAME_LENGTH];
	char size[20];
	float minw, maxw, minh, maxh;
	BStatusBar *sbar;
	BRect rect;
	unsigned char colindex;
	
	rect = win->Bounds();
	rect.Set( XGUTTER, rect.Height(), rect.Width() - XGUTTER, 10 );
	vols.AddItem( vol = new BVolume( id ) );
	win->ResizeBy( 0, height );
	vol->GetName( name );
	sprintf( size, "%.1f", vol->FreeBytes() );
	win->Lock();
	sbar = new BStatusBar( rect, name, name, "K Free" );
	win->AddChild( sbar );
	colindex = index_for_color( 200, 200, 200 );
	sbar->SetViewColor( system_colors()->color_list[colindex] );
	sbar->SetLowColor( system_colors()->color_list[colindex] );
	sbar->SetMaximum( vol->Capacity() / 1024.0 );
	sbar->Progress( vol->Capacity() / 1024.0 );
	sbar->Progress( -(vol->FreeBytes() / 1024.0), 0, size );
	sbar->SetResizingMode( B_FOLLOW_LEFT_RIGHT );
	sbar->SetFlags( B_FULL_UPDATE_ON_RESIZE );
	win->Unlock();
}

void BeFreeApplication::VolumeUnmounted( long id )
{
	long index, count;
	char name[B_OS_NAME_LENGTH];
	bool change = FALSE;
	BView *view;

	win->Lock();
	for( index = 1, count = vols.CountItems(); index < count; index++ )
	{
		if( (index < (count -1)) && ((BVolume *)vols.ItemAt( index ))->ID() == id )
		{
			((BVolume *)vols.ItemAt( index ))->GetName( name );
			view = (BView *)win->ChildAt( index );
			win->RemoveChild( view );
			delete view;
			delete (BVolume *)vols.ItemAt( index );
			vols.RemoveItem( index );
			change = TRUE;
		}
		if( change && (index < (count - 1)) )
		{
			view = win->ChildAt( index );
			view->MoveBy( 0, -height );
		}
	}
	win->Unlock();
	win->ResizeBy( 0, -height );
}

void BeFreeApplication::Pulse()
{
	long index, count;
	BStatusBar *sbar;
	BVolume *vol;
	char size[20];
	system_info sysinfo;
	
	get_system_info( &sysinfo );
	win->Lock();
	count = vols.CountItems();
	sbar = (BStatusBar *)win->ChildAt( 0 );
	sprintf( size, "%.1f", ((sysinfo.max_pages - sysinfo.used_pages) * 4096.0)/1024.0 );
	sbar->Progress( sysinfo.used_pages - sbar->Current(), 0, size );
	for( index = 1; index < count; index++ )
	{
		sbar = (BStatusBar *)win->ChildAt( index );
		vol = (BVolume *)vols.ItemAt( index );
		sprintf( size, "%.1f", vol->FreeBytes() / 1024.0 );
		sbar->Progress( ((vol->Capacity() - vol->FreeBytes()) / 1024.0) - sbar->Current(), 0, size );
	}
	win->Unlock();
}

void BeFreeApplication::ScanVols()
{
	BVolume vol;
	long index = 1;
	
	vol = volume_at( 0 );
	vols.AddItem( 0 );
	while( vol.Error() == B_NO_ERROR )
	{
		vols.AddItem( new BVolume( vol.ID() ) );
		vol = volume_at( index++ );
	}
}

void BeFreeApplication::FreeVols()
{
	long index, count;
	
	for( index = 0, count = vols.CountItems(); index < count; index++ )
	{
		delete (BVolume *)vols.ItemAt( index );
	}
}
